package com.server.alb.models

import jakarta.persistence.*
import jakarta.validation.constraints.NotBlank
import jakarta.validation.constraints.NotNull
import org.springframework.format.annotation.DateTimeFormat
import java.util.*
import kotlin.jvm.Transient

@Entity
@Table(name = "list_bug")
class ListBug {

    @Id
    @Column(name = "bug_id")
    @GeneratedValue(strategy = GenerationType.IDENTITY)
    var id: Long = 0

    @NotNull(message = "Tanggal tidak boleh null")
    @DateTimeFormat(iso = DateTimeFormat.ISO.DATE)
    var date: Date? = null

    @ManyToOne
    @JoinColumn(
        name = "qcId", referencedColumnName = "user_id", foreignKey = ForeignKey(name = "FK_qcBug")
    )
    var qcId: User? = null

    @ManyToOne
    @JoinColumn(
        name = "platform", referencedColumnName = "platform_id", foreignKey = ForeignKey(name = "FK_plaBug")
    )
    @NotNull(message = "Platform jangan blank")
    var platform: Platform? = null

    @NotBlank(message = "Link Good Day tidak boleh empty")
    var link: String? = null

    @NotBlank(message = "Hasil QC jangan blank")
    var result: String? = null

    @NotBlank(message = "Bug tidak boleh kosong")
    @Column(columnDefinition = "varchar(500)")
    var description: String = ""
    var imgLink: String = ""

    @ManyToOne
    @JoinColumn(
        name = "proId", referencedColumnName = "user_id", foreignKey = ForeignKey(name = "FK_proBug")
    )
    @NotNull(message = "Programmer jangan blank")
    var proId: User? = null
    var worked: String = "FALSE"
    var workStat: String = "FALSE"
    var development: String = ""
    var devStat: String = "FALSE"

    @Transient
    var _displayWorked: String? = null

    @Transient
    var _displayWorkStat: String? = null

    @Transient
    var _displayDevStat: String? = null

    @Transient
    var _displayResult: String? = null

    @PostLoad
    fun onLoad() {
        _displayWorked = Status.WorkedType.valueOf(worked).display
        _displayWorkStat = Status.WorkedStatusType.valueOf(workStat).display
        _displayDevStat = Status.DevStatusType.valueOf(devStat).display
        _displayResult = Status.QCResultType.valueOf(result!!).display
    }

    override fun toString(): String {
        return ("Bug [date=$date,description=$description, devStat=$devStat, development=$development, imgLink=$imgLink, link=$link, result=$result, workStat=$workStat, worked=$worked, platform=$platform, proId=$proId, qcId=$qcId]")
    }

}